package net.wanji.utc.service.control.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.google.common.collect.Lists;
import com.hikvision.artemis.sdk.ArtemisHttpUtil;
import com.hikvision.artemis.sdk.config.ArtemisConfig;
import lombok.extern.slf4j.Slf4j;
import net.wanji.utc.common.Result;
import net.wanji.utc.common.baseentity.BaseCrossInfo;
import net.wanji.utc.common.exception.ControlException;
import net.wanji.utc.common.typeenum.BasicEnum;
import net.wanji.utc.common.typeenum.DateStyle;
import net.wanji.utc.dto.hk.DownloadPlanSpandsDTO;
import net.wanji.utc.dto.hk.DownloadScheduleDTO;
import net.wanji.utc.dto.hk.DownloadTimeSpandsDTO;
import net.wanji.utc.mapper.*;
import net.wanji.utc.po.*;
import net.wanji.utc.service.control.ControlCommandService;
import net.wanji.utc.service.runninginfo.HkLightsStatusService;
import net.wanji.utc.util.FieldUtil;
import net.wanji.utc.util.PathUtil;
import net.wanji.utc.vo.*;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

import static net.wanji.utc.common.ResultEnum.BODY_NOT_MATCH;
import static net.wanji.utc.common.constant.Constants.*;

/**
 * @author wuxiaokai
 * @date 2022/11/21 9:51:47
 */
@Slf4j
@Service("hkControlCommandService")
public class HKControlCommandServiceImpl implements ControlCommandService {

    @Resource
    private HkLightsStatusService hkLightsStatusService;

    @Resource
    private ArtemisConfig artemisConfig;

    @Resource
    private CrossPhaseMapper crossPhaseMapper;

    @Resource
    private CrossInfoMapper crossInfoMapper;

    @Resource
    private CrossSchemeMapper crossSchemeMapper;

    @Resource
    private CrossPlanMapper crossPlanMapper;

    @Resource
    private CrossSectionMapper crossSectionMapper;

    @Resource
    private CrossSchedulesMapper crossSchedulesMapper;

    @Resource
    private ManufacturerInfoMapper manufacturerInfoMapper;

    @Override
    public <T> Result<T> schemeSend(SchemeSendVO schemeSendVO) throws Exception {
        // 厂商 HK
        ManufacturerInfoPO manufacturerInfoPO = checkCrossCode(schemeSendVO.getCrossCode());
        if (manufacturerInfoPO.getCode().equals(BasicEnum.ManufacturerEnum.HK.getCode())) {

            // 1、组装参数, 下发海康信号机
            DownloadPlanSpandsDTO dto = new DownloadPlanSpandsDTO();
            // 将路口code转换为信号机code
            //dto.setCrossCode(schemeSendVO.getCrossCode());
            dto.setCrossCode(manufacturerInfoPO.getName());
            List<DownloadPlanSpandsDTO.Pattern> patternList = new ArrayList<>();
            schemeSendVO.getPatternList().forEach(pattern -> {
                DownloadPlanSpandsDTO.Pattern pattern1 = new DownloadPlanSpandsDTO.Pattern();
                pattern1.setPatternNo(pattern.getPatternNo());
                pattern1.setPatternName(pattern.getPatternName());
                pattern1.setCoordPhase(pattern.getCoordPhase());
                pattern1.setOffset(pattern.getOffset());
                pattern1.setCycle(pattern.getCycle());
                List<DownloadPlanSpandsDTO.Pattern.Ring> rings = new ArrayList<>();
                pattern.getRings().forEach(ring -> {
                    DownloadPlanSpandsDTO.Pattern.Ring ring1 = new DownloadPlanSpandsDTO.Pattern.Ring();
                    ring1.setRingNo(ring.getRingNo());
                    List<DownloadPlanSpandsDTO.Pattern.Ring.Phase> phaseList = new ArrayList<>();
                    ring.getPhaseList().forEach(phase -> {
                        DownloadPlanSpandsDTO.Pattern.Ring.Phase phase1 = new DownloadPlanSpandsDTO.Pattern.Ring.Phase();
                        phase1.setPhaseNo(phase.getPhaseNo());
                        phase1.setPhaseName(phase.getPhaseName());
                        phase1.setPhaseLength(phase.getPhaseTime());
                        phase1.setPhaseSequence(phase.getSort());
                        phase1.setLanes(Lists.newArrayList());
                        phaseList.add(phase1);
                    });
                    ring1.setPhaseList(phaseList);
                    rings.add(ring1);
                });
                pattern1.setRings(rings);
                patternList.add(pattern1);
            });
            dto.setPatternList(patternList);
            // 下发信号机方案
            Map<String, Object> fieldMap = FieldUtil.fieldMethod(dto);
            Map<String, String> path = PathUtil.getPathMapByApiCode("downloadPlanSpands");
            String strResult = ArtemisHttpUtil.doPostStringArtemis(artemisConfig,
                    path,
                    JSON.toJSONString(fieldMap), null, null, "application/json", null);

            JSONObject obj = JSON.parseObject(strResult);
            if (obj.get(HK_CODE_KEY).equals(HK_SUCCESS_STRING_CODE)) {
                // 入库操作
                List<CrossSchemePO> crossSchemePOList = new ArrayList<>();
                schemeSendVO.getPatternList().forEach(pattern -> {
                    CrossSchemePO crossSchemePO = new CrossSchemePO();
                    crossSchemePO.setSchemeNo(pattern.getPatternNo());
                    crossSchemePO.setName(pattern.getPatternName());
                    crossSchemePO.setCrossId(schemeSendVO.getCrossCode());
                    crossSchemePO.setCycle(Integer.parseInt(pattern.getCycle()));
                    crossSchemePO.setCoordPhase(Integer.parseInt(pattern.getCoordPhase()));
                    crossSchemePO.setOffset(Integer.parseInt(pattern.getOffset()));
                    crossSchemePO.setSource(2);
                    crossSchemePO.setIsDeleted(0);
                    crossSchemePOList.add(crossSchemePO);

                    pattern.getRings().forEach(ring -> {
                        List<CrossPhasePO> crossPhasePOList = new ArrayList<>();
                        ring.getPhaseList().forEach(phase -> {
                            CrossPhasePO crossPhasePO = new CrossPhasePO();
                            crossPhasePO.setPhaseNo(phase.getPhaseNo());
                            crossPhasePO.setName(phase.getPhaseName());
                            crossPhasePO.setSort(Integer.parseInt(phase.getSort()));
                            crossPhasePO.setCrossId(schemeSendVO.getCrossCode());
                            crossPhasePO.setPlanId(Integer.parseInt(pattern.getPatternNo()));
                            crossPhasePO.setRingNo(Integer.parseInt(ring.getRingNo()));
                            crossPhasePO.setControlMode(Integer.parseInt(phase.getControlMode()));
                            crossPhasePO.setPhaseTime(Integer.parseInt(phase.getPhaseTime()));
                            crossPhasePO.setGreenTime(Integer.parseInt(phase.getGreenTime()));
                            crossPhasePO.setGreenFlashTime(Integer.parseInt(phase.getGreenFlashTime()));
                            crossPhasePO.setPedFlashTime(Integer.parseInt(phase.getPedFlashTime()));
                            crossPhasePO.setYellowTime(Integer.parseInt(phase.getYellowTime()));
                            crossPhasePO.setRedTime(Integer.parseInt(phase.getRedTime()));
                            crossPhasePO.setMinGreenTime(Integer.parseInt(phase.getMinGreenTime()));
                            crossPhasePO.setMaxGreenTime(Integer.parseInt(phase.getMaxGreenTime()));
                            crossPhasePOList.add(crossPhasePO);
                        });
                        // 入库路口相位表
                        crossPhaseMapper.insertBatch(crossPhasePOList);
                    });

                });
                // 入库信号方案表
                crossSchemeMapper.insertBatch(crossSchemePOList);

                return Result.success("下发信号机方案成功");
            }
            return Result.error(obj.getString("msg"));
        } else {
            return null;
        }

    }

    @Override
    public <T> Result<T> planSend(PlanSendVO planSendVO) throws Exception {
        // 厂商 HK
        ManufacturerInfoPO manufacturerInfoPO = checkCrossCode(planSendVO.getCrossCode());
        if (manufacturerInfoPO.getCode().equals(BasicEnum.ManufacturerEnum.HK.getCode())) {
            // 1、下发信号机计划拼接参数
            DownloadTimeSpandsDTO downloadTimeSpandsDTO = new DownloadTimeSpandsDTO();
            List<DownloadTimeSpandsDTO.Data> dataList = new ArrayList<>();
            DownloadTimeSpandsDTO.Data data = new DownloadTimeSpandsDTO.Data();
            //data.setCrossCode(planSendVO.getCrossCode());
            data.setCrossCode(manufacturerInfoPO.getName());
            List<DownloadTimeSpandsDTO.Data.Plan> planList = new ArrayList<>();
            planSendVO.getPlanList().forEach(plan -> {
                DownloadTimeSpandsDTO.Data.Plan plan1 = new DownloadTimeSpandsDTO.Data.Plan();
                plan1.setPlanNo(plan.getPlanNo());
                List<DownloadTimeSpandsDTO.Data.Plan.Section> sectionList = new ArrayList<>();
                plan.getSectionList().forEach(section -> {
                    DownloadTimeSpandsDTO.Data.Plan.Section section1 = new DownloadTimeSpandsDTO.Data.Plan.Section();
                    section1.setTimeSecNo(section.getSectionNo());
                    section1.setPatternNo(section.getPatternNo());
                    section1.setControlType(section.getControlMode());
                    section1.setBeginTime(section.getBeginTime());
                    sectionList.add(section1);
                });
                plan1.setSectionList(sectionList);
                planList.add(plan1);
            });
            data.setPlanList(planList);
            dataList.add(data);
            downloadTimeSpandsDTO.setData(dataList);

            // 1、下发信号机计划
            Map<String, Object> fieldMap = FieldUtil.fieldMethod(downloadTimeSpandsDTO);
            Map<String, String> path = PathUtil.getPathMapByApiCode("downloadTimeSpands");
            String strResult = ArtemisHttpUtil.doPostStringArtemis(artemisConfig,
                    path,
                    JSON.toJSONString(fieldMap), null, null, "application/json", null);
            JSONObject obj = JSON.parseObject(strResult);
            if (obj.get(HK_CODE_KEY).equals(HK_SUCCESS_STRING_CODE)) {
                // 2、入库操作
                List<CrossPlanPO> crossPlanPOList = new ArrayList<>();
                planSendVO.getPlanList().forEach(plan -> {
                    CrossPlanPO crossPlanPO = new CrossPlanPO();
                    crossPlanPO.setPlanNo(plan.getPlanNo());
                    crossPlanPO.setName(plan.getPlanDescribe());
                    crossPlanPO.setCrossId(planSendVO.getCrossCode());
                    crossPlanPOList.add(crossPlanPO);
                    List<CrossSectionPO> crossSectionPOList = new ArrayList<>();
                    plan.getSectionList().forEach(section -> {
                        CrossSectionPO crossSectionPO = new CrossSectionPO();
                        crossSectionPO.setSectionNo(section.getSectionNo());
                        crossSectionPO.setStartTime(section.getBeginTime());
                        crossSectionPO.setEndTime(section.getEndTime());
                        crossSectionPO.setCrossId(planSendVO.getCrossCode());
                        crossSectionPO.setPlanId(Integer.parseInt(plan.getPlanNo()));
                        crossSectionPO.setControlMode(Integer.parseInt(section.getControlMode()));
                        crossSectionPO.setScemeId(Integer.parseInt(section.getPatternNo()));
                        crossSectionPOList.add(crossSectionPO);
                    });
                    // 入库路口时段表
                    crossSectionMapper.insertBatch(crossSectionPOList);
                });
                // 入库路口计划表
                crossPlanMapper.insertBatch(crossPlanPOList);

                return Result.success("下发信号机计划成功");
            }
            return Result.error(obj.getString("msg"));
        } else {
            // todo else
            return null;
        }
    }

    @Override
    public <T> Result<T> scheduleSend(ScheduleSendVO param) throws Exception {
        // 厂商 HK
        ManufacturerInfoPO manufacturerInfoPO = checkCrossCode(param.getCrossCode());
        if (manufacturerInfoPO.getCode().equals(BasicEnum.ManufacturerEnum.HK.getCode())) {
            // 1、通过获取当前运行的日期数据,获取最新的日期方案id
            JSONArray body = new JSONArray();
            //body.add(param.getCrossCode());
            body.add(manufacturerInfoPO.getName());
            Map<String, String> path = PathUtil.getPathMapByApiCode("uploadDate");
            String strResult = ArtemisHttpUtil.doPostStringArtemis(artemisConfig,
                    path,
                    body.toString(), null, null, "application/json", null);
            JSONObject object = JSON.parseObject(strResult);
            // 日期 id
            int id = 1;
            if (object.get(HK_CODE_KEY).equals(HK_SUCCESS_STRING_CODE)) {
                // 获取日期列表
                JSONArray schedulesArray = object.getJSONArray("data").getJSONObject(0).getJSONArray("schedules");
                JSONObject scheduleObj = schedulesArray.getJSONObject(schedulesArray.size() - 1);
                id = scheduleObj.getInteger("id");
            }
            // 下发信号机日期拼接参数
            DownloadScheduleDTO dto = new DownloadScheduleDTO();
            //dto.setCrossCode(param.getCrossCode());
            dto.setCrossCode(manufacturerInfoPO.getName());
            List<DownloadScheduleDTO.Schedule> schedules = new ArrayList<>();
            for (ScheduleSendVO.Schedule schedule : param.getSchedules()) {
                List<ScheduleSendVO.Schedule.Week> scheduleWeeks = schedule.getWeeks();
                List<ScheduleSendVO.Schedule.SpecialDay> scheduleDays = schedule.getSpecialDays();
                if (null != scheduleWeeks && scheduleWeeks.size() > 0) {
                    // 去重，获取所有的运行方案号
                    Set<Integer> planNos = new HashSet<>();
                    scheduleWeeks.forEach(week -> planNos.add(week.getPlanNo()));
                    if (null != scheduleDays && scheduleDays.size() > 0) {
                        scheduleDays.forEach(specialDay -> planNos.add(specialDay.getPlanNo()));
                    }
                    int num = 0;
                    for (Integer planNo : planNos) {
                        DownloadScheduleDTO.Schedule schedule1 = new DownloadScheduleDTO.Schedule();
                        // scheduleNo 日期号 ，支持返回【1~128】
                        if (id + planNos.size() > 128) {
                            schedule1.setScheduleNo(128 - planNos.size() - (++num));
                        } else {
                            schedule1.setScheduleNo(id + (++num));
                        }
                        // 0 说明 dates 内的日期是日期集合，1 说明是dates 内的数据是日期范围
                        schedule1.setIsPeriod(0);
                        // 特殊日期列表
                        List<String> dates = new ArrayList<>();
                        schedule.getSpecialDays().forEach(specialDay -> {
                            if (specialDay.getPlanNo().equals(planNo)) {
                                dates.add(specialDay.getDateStr());
                            }
                        });
                        schedule1.setDates(dates);
                        // 星期列表
                        List<String> weeks = new ArrayList<>();
                        schedule.getWeeks().forEach(week -> {
                            if (week.getPlanNo().equals(planNo)) {
                                weeks.add(String.valueOf(week.getWeekNum()));
                            }
                        });
                        schedule1.setWeeks(weeks);
                        schedule1.setPlanNo(planNo);
                        schedules.add(schedule1);
                    }
                }
            }
            dto.setSchedules(schedules);
            // 2、下发时间表
            Map<String, Object> fieldMap = FieldUtil.fieldMethod(dto);
            for (DownloadScheduleDTO.Schedule schedule : dto.getSchedules()) {
                List<Map<String, Object>> mapSchedules = (List<Map<String, Object>>) fieldMap.get("schedules");
                for (Map<String, Object> mapSchedule : mapSchedules) {
                    if (mapSchedule.get("planNo").equals(schedule.getPlanNo())) {
                        mapSchedule.put("weeks", schedule.getWeeks());
                        mapSchedule.put("dates", schedule.getDates());
                        break;
                    }
                }
            }
            Map<String, String> downloadSchedulePath = PathUtil.getPathMapByApiCode("downloadTimeSpands");
            String updateResult = ArtemisHttpUtil.doPostStringArtemis(artemisConfig,
                    downloadSchedulePath,
                    JSON.toJSONString(fieldMap), null, null, "application/json", null);
            JSONObject obj = JSON.parseObject(updateResult);
            if (obj.get(HK_CODE_KEY).equals(HK_SUCCESS_STRING_CODE)) {

                // 1、入库操作
                SimpleDateFormat sdf = new SimpleDateFormat(DateStyle.YYYY_MM_DD.getValue());
                ArrayList<CrossSchedulesPO> crossSchedulesPOS = new ArrayList<>();
                param.getSchedules().forEach(schedule -> {
                    schedule.getWeeks().forEach(week -> {
                        CrossSchedulesPO schedulesPO = new CrossSchedulesPO();
                        if (week.getWeekNum() >= 6) {
                            schedulesPO.setName("周末");
                        } else {
                            schedulesPO.setName("工作日");
                        }
                        schedulesPO.setCrossId(param.getCrossCode());
                        schedulesPO.setPlanId(week.getPlanNo());
                        schedulesPO.setWeek(week.getWeekNum());
                        schedulesPO.setSpecialDate(null);
                        crossSchedulesPOS.add(schedulesPO);
                    });
                    schedule.getSpecialDays().forEach(specialDay -> {
                        CrossSchedulesPO schedulesPO = new CrossSchedulesPO();
                        schedulesPO.setName("特殊日期");
                        schedulesPO.setCrossId(param.getCrossCode());
                        schedulesPO.setPlanId(specialDay.getPlanNo());
                        schedulesPO.setWeek(null);
                        try {
                            schedulesPO.setSpecialDate(sdf.parse(specialDay.getDateStr()));
                        } catch (ParseException e) {
                            schedulesPO.setSpecialDate(null);
                        }
                        crossSchedulesPOS.add(schedulesPO);
                    });
                });
                // 入库时间表
                crossSchedulesMapper.insertBatch(crossSchedulesPOS);

                return Result.success("时间表下发成功");
            }
            return Result.error(obj.getString("msg"));
        } else {
            // todo else
            return null;
        }
    }

    @Override
    public <T> Result<T> lockControl(ControlCommandVO commandVO) throws Exception {
        ManufacturerInfoPO manufacturerInfoPO = checkCrossCode(commandVO.getCrossCode());
        if (manufacturerInfoPO.getCode().equals(BasicEnum.ManufacturerEnum.HK.getCode())) {
            if (commandVO.getCommand().equals(TRUE)) {
                Map<String, Object> param = new HashMap<String, Object>() {
                    {
                        //put("crossCode", commandVO.getCrossCode());
                        put("crossCode", manufacturerInfoPO.getName());
                        put("controlType", LOCK_RUNNING_CONTROL);
                    }
                };
                // 控制命令为灯态锁定时，设置相关车道参数信息
                //List<SignalRunring> runrings = getSignalInfoVos(commandVO.getCrossCode()).get(0).getRunrings();
                // HK 为单环
                // 获取当前运行方案的相位id
                LightsStatusVO statusVO = getSignalInfoVos(manufacturerInfoPO.getName()).get(0);
                Integer phaseId = Integer.valueOf(statusVO.getPhaseId());
                // 获取某一路口下海康相位和车道关系
                //Map<Integer, JSONArray> laneArrayMap = getPhaseRelationLanes(commandVO.getCrossCode());
                Map<Integer, JSONArray> laneArrayMap = getPhaseRelationLanes(manufacturerInfoPO.getName());
                JSONArray jsonArray = laneArrayMap.get(phaseId);
                List<Map<String, Object>> laneList = new ArrayList<>();
                for (int i = 0; i < jsonArray.size(); i++) {
                    JSONObject jsonObject = jsonArray.getJSONObject(i);
                    Map<String, Object> laneMap = new HashMap<>();
                    laneMap.put("laneNo", jsonObject.getInteger("laneNo"));
                    laneMap.put("turn", jsonObject.getInteger("turn"));
                    laneMap.put("direction", jsonObject.getInteger("direction"));
                    laneList.add(laneMap);
                }
                param.put("laneInfos", laneList);
                param.put("controlNo", 0);
                // 控制时长，单位 s，0 则持续控制
                param.put("duration", Objects.isNull(commandVO.getDuration()) || commandVO.getDuration() == 0 ? 999 : commandVO.getDuration());
                String strResult = ArtemisHttpUtil.doPostStringArtemis(artemisConfig,
                        PathUtil.getPathMapByApiCode("setSignalControl"),
                        JSON.toJSONString(param), null, null, "application/json", null);
                JSONObject obj = JSON.parseObject(strResult);
                if (HK_SUCCESS_STRING_CODE.equals(obj.get(HK_CODE_KEY))) {
                    return Result.success("相位锁定成功");
                }
                return Result.error(obj.getString("msg"));
            } else {
                //return controlSignal(commandVO.getCrossCode(), UNLOCK_RUNNING_CONTROL);
                return controlSignal(manufacturerInfoPO.getName(), UNLOCK_RUNNING_CONTROL);
            }
        } else {
            // todo else
            return null;
        }
    }

    /**
     * 获取某一路口下海康相位和车道关系
     *
     * @param crossCode 信号机唯一编号
     * @return {@link Map}<{@link Integer}, {@link JSONArray}> 相位与车道集合
     * @throws Exception 异常
     */
    private Map<Integer, JSONArray> getPhaseRelationLanes(String crossCode) throws Exception {
        Map<String, String> path = PathUtil.getPathMapByApiCode("getPhaseRlatLanes");
        Map<String, String> queryMap = new HashMap<>();
        queryMap.put("crossCode", crossCode);
        String strResult = ArtemisHttpUtil.doPostStringArtemis(artemisConfig, path, null, queryMap,
                null, "application/json", null);
        JSONObject obj = JSON.parseObject(strResult);
        Map<Integer, JSONArray> laneArrayMap = new HashMap<>();
        if (HK_SUCCESS_STRING_CODE.equals(obj.get(HK_CODE_KEY))) {
            JSONObject data = obj.getJSONObject("data");
            JSONArray phaseList = data.getJSONArray("phaseList");
            for (int i = 0; i < phaseList.size(); i++) {
                JSONObject lanesObject = phaseList.getJSONObject(i);
                Integer phaseNo = lanesObject.getInteger("phaseNo");
                JSONArray laneList = lanesObject.getJSONArray("lanes");
                // 存入相位与车道集合
                laneArrayMap.put(phaseNo, laneList);
            }
        } else {
            throw new ControlException(obj.getString("msg"));
        }
        return laneArrayMap;
    }

    @Override
    public <T> Result<T> stepControl(String code, Integer command, Integer stepNum) throws Exception {
        ManufacturerInfoPO manufacturerInfoPO = checkCrossCode(code);
        if (manufacturerInfoPO.getCode().equals(BasicEnum.ManufacturerEnum.HK.getCode())) {
            if (command.equals(TRUE)) {

                Map<String, Object> param = new HashMap<String, Object>() {
                    {
                        //put("crossCode", code);
                        put("crossCode", manufacturerInfoPO.getName());
                        put("controlType", STEP_CONTROL);
                    }
                };
                int controlNo = 0;
                if (0 != stepNum) {
                    //根据当前实时灯态获取运行的相位
                    //LightsStatusVO lightsStatusVO = getSignalInfoVos(code).get(0);
                    LightsStatusVO lightsStatusVO = getSignalInfoVos(manufacturerInfoPO.getName()).get(0);
                    //暂时按单环处理
                    String phaseId = lightsStatusVO.getPhaseId();
                    //获取方案数据列表
                    List<CrossPhasePO> baseCrossPhasePlanList = crossPhaseMapper.selectByCrossIdAndPlanId(lightsStatusVO.getCode(), lightsStatusVO.getPhasePlanId());
                    Map<String, List<CrossPhasePO>> groupByPhase = baseCrossPhasePlanList.stream().collect(Collectors.groupingBy(CrossPhasePO::getPhaseNo));
                    //根据相位获取当前相序
                    int newPhaseOrderId = 0;
                    for (String key : groupByPhase.keySet()) {
                        CrossPhasePO crossPhase = groupByPhase.get(key).get(0);
                        if (crossPhase.getPhaseNo().equals(phaseId)) {
                            newPhaseOrderId = (crossPhase.getSort() + stepNum) % groupByPhase.size();
                        }
                    }
                    //根据相序获取相位
                    for (String key : groupByPhase.keySet()) {
                        CrossPhasePO crossPhase = groupByPhase.get(key).get(0);
                        if (newPhaseOrderId == crossPhase.getSort()) {
                            controlNo = Integer.parseInt(crossPhase.getPhaseNo());
                        }
                    }
                }
                param.put("controlNo", controlNo);
                String strResult = ArtemisHttpUtil.doPostStringArtemis(artemisConfig,
                        PathUtil.getPathMapByApiCode("setSignalControl"),
                        JSON.toJSONString(param), null, null, "application/json", null);
                JSONObject obj = JSON.parseObject(strResult);
                if (HK_SUCCESS_STRING_CODE.equals(obj.get(HK_CODE_KEY))) {
                    return Result.success();
                }
                return Result.error(obj.getString("msg"));
            } else {
                //return controlSignal(code, CANCEL_STEP_CONTROL);
                return controlSignal(manufacturerInfoPO.getName(), CANCEL_STEP_CONTROL);
            }
        } else {
            // todo else
            return null;
        }
    }

    @Override
    public <T> Result<T> setSignalControl(String code, Integer command, Integer commandType) throws Exception {
        ManufacturerInfoPO manufacturerInfoPO = checkCrossCode(code);
        if (manufacturerInfoPO.getCode().equals(BasicEnum.ManufacturerEnum.HK.getCode())) {
            Integer runningMode = command.equals(TRUE) ? commandType : NORMAL_RUNNING_CONTROL;
            //return controlSignal(code, runningMode);
            return controlSignal(manufacturerInfoPO.getName(), runningMode);
        } else {
            // todo else
            return null;
        }
    }

    @Override
    public <T> Result<T> recoverSchedule(String code) throws Exception {
        ManufacturerInfoPO manufacturerInfoPO = checkCrossCode(code);
        if (manufacturerInfoPO.getCode().equals(BasicEnum.ManufacturerEnum.HK.getCode())) {
            //return controlSignal(code, NORMAL_RUNNING_CONTROL);
            return controlSignal(manufacturerInfoPO.getName(), NORMAL_RUNNING_CONTROL);
        } else {
            // todo else
            return null;
        }
    }

    @Override
    public <T> Result<T> phaseTimingSend(PhaseTimingSendVO phaseTimingSendVO) throws Exception {
        ManufacturerInfoPO manufacturerInfoPO = checkCrossCode(phaseTimingSendVO.getCrossCode());
        if (manufacturerInfoPO.getCode().equals(BasicEnum.ManufacturerEnum.HK.getCode())) {
            // 将路口code转换为信号机code
            phaseTimingSendVO.setCrossCode(manufacturerInfoPO.getName());
            Map<String, Object> param = FieldUtil.fieldMethod(phaseTimingSendVO);
            Map<String, String> path = PathUtil.getPathMapByApiCode("setPhaseRlatLanes");
            String strResult = ArtemisHttpUtil.doPostStringArtemis(artemisConfig,
                    path,
                    JSON.toJSONString(param), null, null, "application/json", null);
            JSONObject obj = JSON.parseObject(strResult);
            if (HK_SUCCESS_STRING_CODE.equals(obj.get(HK_CODE_KEY))) {
                return Result.success();
            }
            return Result.error(obj.getString("msg"));
        } else {
            // todo else
            return null;
        }
    }

    @Override
    public <T> Result<T> phaseDiffSend() {
        return null;
    }

    /**
     * @param code        信号机编号
     * @param runningMode 0-多时段控制模式 1-关灯、2-黄闪、3-全红、11-取消步进控制、12-灯态锁定、13-灯态解锁
     */
    private <T> Result<T> controlSignal(String code, Integer runningMode) throws Exception {
        //执行恢复时间表方案命令
        if (NORMAL_RUNNING_CONTROL.equals(runningMode) || CANCEL_STEP_CONTROL.equals(runningMode)) {
            //根据实时灯态获取当前运行模式
            List<BaseCrossInfo> baseSignals = new ArrayList<>();
            BaseCrossInfo baseSignal = new BaseCrossInfo();
            baseSignal.setCode(code);
            baseSignal.setManufacturerCode(BasicEnum.ManufacturerEnum.HK.getCode());
            baseSignals.add(baseSignal);
            LightsStatusVO lightsStatusVO = hkLightsStatusService.getHkLightsStatus(baseSignals).get(0);
            //如果当前模式已经是定周期，就不在执行恢复时间表方案指令
            if (FIXED_CYCLE_MODE.equals(Integer.valueOf(lightsStatusVO.getRunMode()))) {
                throw new ControlException("当前控制模式是定周期控制，请勿下发恢复命令");
            }
        }
        Map<String, Object> param = new HashMap<String, Object>() {
            {
                put("crossCode", code);
                put("controlType", runningMode);
                put("controlNo", 0);
                //控制时长，单位 s，0 则持续控制
                put("duration", 0);
            }
        };
        String strResult = ArtemisHttpUtil.doPostStringArtemis(artemisConfig,
                PathUtil.getPathMapByApiCode("setSignalControl"),
                JSON.toJSONString(param), null, null, "application/json", null);
        JSONObject obj = JSON.parseObject(strResult);
        if (HK_SUCCESS_STRING_CODE.equals(obj.get(HK_CODE_KEY))) {
            return Result.success();
        }
        return Result.error(obj.getString("msg"));
    }

    /**
     * 根据信号机获取灯态列表
     */
    private List<LightsStatusVO> getSignalInfoVos(String code) {
        List<BaseCrossInfo> baseSignals = new ArrayList<>();
        BaseCrossInfo baseSignal = new BaseCrossInfo();
        baseSignal.setCode(code);
        baseSignal.setManufacturerCode(BasicEnum.ManufacturerEnum.HK.getCode());
        baseSignals.add(baseSignal);
        return hkLightsStatusService.getHkLightsStatus(baseSignals);
    }

    /**
     * 通用方法-检查路口id
     * 查询出信号机id
     */
    private ManufacturerInfoPO checkCrossCode(String crossCode) {
        // 根据路口id查询
        CrossInfoPO crossInfoPo = crossInfoMapper.selectByPrimaryKey(crossCode);
        if (crossInfoPo == null) {
            throw new ControlException(BODY_NOT_MATCH.getResultCode(), "参数错误，路口ID不正确。");
        }
        // 厂商 HK
        ManufacturerInfoPO manufacturerInfoPO = manufacturerInfoMapper.selectById(crossInfoPo.getManufacturerId());
        // 路口code不是信号机code，此处为了方便，利用ManufacturerInfoPO的name属性做一下存储
        manufacturerInfoPO.setName(crossInfoPo.getCode());
        return manufacturerInfoPO;
    }
}
